indexing

   title:       "COMMERCIAL_FORMAT usage example";
   cluster:     "format/commerce/examples/pic [format/commerce/examples/pic]";
   project:     "The Universal Simple Formatting Library (USFL)";
   copyright:   "SIG Computer, 1995";
   author:      "Andy Malakov  (andy@eiffel.ru) and";
   original:    08,Sep,1995;
   version:     1.0;
   last_change:
   done_at:     "SIG Computer (info@eiffel.ru)";
   extrnl_name: "pic.e"

class PIC
      -- Class illustrate COMMERCIAL_FORMAT usage.
   inherit
      COMMERCIAL_FORMAT
         rename make as cformat_make
         end
      EXCEPTION

   creation
      make

   feature

      make is
         local
            i : INTEGER
         do
            cformat_make

   ------------------------------------------------------------
   -- You may mix-up several types in format string
   -- argument must conform to corresponding typechar
   -- If picture character repeats more that once you
   -- can use brief form e.g. 99999999 is the same as 9(8)
   ------------------------------------------------------------

            printf ( "!9999 !9.999 !XXXX %N", <<123, 1.23, "abc">> )
            printf ( "!9(4) !9.9(3) !X(2)X(2)%N", <<123, 1.23, "abc">> )
            printf ( "!9(10)%N", <<1>> ) -- !9999999999
            printf ( "!9(1)%N", <<1>> )   -- same as "!9"

   ------------------------------------------------------------
   -- You may use formatting options in square brackets
   -- such as [BLANK WHEN ZERO] or its shortform [BZ]
   ------------------------------------------------------------
            printf ( "!9(10)[BZ]%N", <<1>> ) -- !9999999999
            printf ( "!9999[BLANK WHEN ZERO]%N", <<123>> )
            printf ( "!9[BZ]999%N", <<123>> )

   ------------------------------------------------------------
   -- You can enclose picture in brackets to sepatate it from
   -- other characters in format string
   ------------------------------------------------------------
            printf ( "!(99(9))999", <<1>> ) -- 9999999999+999
            printf ( "(!9)", <<1>> )
            printf ( "!(9)995%N", <<1>> ) -- print 1995

   ------------------------------------------------------------
   -- You can mix COMMERCIAL and SCIENTIFIC format specification
   ------------------------------------------------------------
            printf ( "Mixed !9(1) !d%N", <<1,1>> )
            printf ( "Mixed !9(1) !s%N", <<1,"">> )

   ------------------------------------------------------------
   -- You can set picture format as default for all objects
   -- with conformad type.
   ------------------------------------------------------------

            set_default_format ( "!99999" )
            print ( 123 )
            set_default_format ( "!-d" )
            print ( 123 )

            set_default_format ( "!99999.9" )
            print ( 0.123 )
            set_default_format ( "!f" )
            print ( 0.123 )
            printf ( "Is it [!99999.9]%N" , << 0.123 >> )

----------------------------------------------------------------------
-- Hints
----------------------------------------------------------------------

            printf ( "%N******** Now some bugs *********%N" , << >> )

----------------------------------------------------------------------
-- Errors (remarks show symbols caused error)
----------------------------------------------------------------------

            try_printf ( "! 999", <<123>> )        -- ' '

            try_printf ( "!()...",  <<1>> )        -- '()'
            try_printf ( "!)...",   <<1>> )        -- ')'
            try_printf ( "!9[]...", <<1>> )        -- '[]'
            try_printf ( "!]",      <<1>> )        -- ']'

            try_printf ( "!(91)",   <<1>> )        -- '1'
            try_printf ( "!(99",   <<1>> )         -- ')'
            try_printf ( "!9[[",   <<1>> )         -- '['

            try_printf ( "!(9) !9", <<1>> )      -- number of arguments
            try_printf ( "!(9)",    <<"abc">> )  -- type of argument
            try_printf ( "!99",     <<123>> )    -- number not fit

            try_printf ( "!9.9string",   <<3.14>> ) -- error !9.9s & 3.14
            printf ( "!(9.9)string", <<3.14>> )    -- ok !9.9

            -- repeatance counter must be a positive number
            try_printf ( "!9999(-1)", <<123>> )   -- '(-1)'

            -- empty picture before repeatance
            try_printf ( "!(1)", <<1>> )          -- (1)

            -- bad repeatance number
            try_printf ( "!9(3(3))", <<1>> )          -- (3(3))
            try_printf ( "!(9(3(3))9)9", <<1>> )
            try_printf ( "!9(3(3", <<1>> )
            try_printf ( "!(9(3(3", <<1>> )

            -- twice repeatance (unsupported)
            try_printf ( "!9(1)(2)", <<1>> )      -- (1)(2)

            -- twice repeatance (unsupported)
            try_printf ( "!9(1)[BZ](2)", <<1>> )  -- (1)(2)


         end -- make

----------------------------------------------------------------------
      try_printf ( format : STRING, para : ARRAY [ ANY ] ) is
            -- ignore exceptions
         local
            failed : BOOLEAN
            f : STRING
         do
            if not failed then printf ( format, para ) end
         rescue
            std_out.put_string ("PRINTF execution failed (Training exception).%N")
            failed := true
            retry
         end -- try_printf
----------------------------------------------------------------------
      try_print ( para : ANY ) is
            -- ignore exceptions
         local
            failed : BOOLEAN
            f : STRING
         do
            if not failed then print ( para ) end
         rescue
            std_out.put_string ("PRINT execution failed (Training exception).%N")
            failed := true
            retry
         end -- try_printf

end -- class PIC
